/******************************************
 * ptsc_hdr.c, @format.new-line     crlf  * 
 *             @format.use-tabs     true  * 
 *             @format.tab-size     3		* 
 *					@format.indent-size	3		* 
 *					@format.line-length	79		* 
 *														* 
 * This source file has been released to	*
 * the Public Domain by Rob Swindell.		*
 *                                        *
 * For more information, go to 				*
 * http://www.synchro.net/ptsc_hdr.html	* 
 ******************************************/
	 
/************/
/* Includes */
/************/

#include <stdio.h>		/* fprintf, feof, fgets, etc. prototypes		*/
#include <ctype.h>		/* tolower definition								*/
#include <stdlib.h>		/* strtol prototype									*/
#include <string.h>		/* strcpy, strcmp, etc. prototypes				*/

/*************/
/* Constants */
/*************/

#define PTSC_MAX_FILE_OFFSET		3000
#define PTSC_MAX_LINES				60
#define PTSC_MAX_LINE_LENGTH		160
#define PTSC_MAX_VALUES				40

const char* ptsc_header_token	=	"@format.";

/* Supported variables */
enum {
	PTSC_TAB_SIZE,
	PTSC_TAB_STOPS,
	PTSC_INDENT_SIZE,
	PTSC_LINE_LENGTH,
	PTSC_NEW_LINE,
	PTSC_USE_TABS,
	PTSC_TOTAL_VARS
	};

/*********************************************************************/
/* Reads text from "stdin", parsing any valid PT/SC headers present	*/
/*********************************************************************/
void main()
{
	char	line[512];								/* line read from file		*/
	char	hdr[PTSC_MAX_LINE_LENGTH + 1];	/* possible header(s)		*/
	char*	p;											/* text pointer				*/
	char*	np;										/* new text pointer			*/
	int	i;											/* format variable index	*/
	int	j;											/* variable value index		*/
	int	k;											/* keyword index				*/
	long	lines_read = 0;						/* lines read from file		*/
	struct
		{
		char*	name;									/* variable name				*/
		int	val_specified;						/* total values specified	*/
		int	val[PTSC_MAX_VALUES];			/* variable value array		*/
		} var[PTSC_TOTAL_VARS] =				/* format variable array	*/
			{
				{ "tab-size",		0, {0} },
				{ "tab-stops",		0, {0} },
				{ "indent-size",	0, {0} },
				{ "line-length",	0, {0} },
				{ "new-line",		0, {0} },
				{ "use-tabs",		0, {0} }
			};
	struct
		{
		char*	token;								/* keyword token				*/
		int	value;								/* keyword value				*/
		} keyword[] =
			{
				{ "true",	1 		},
				{ "false",	0 		},
				{ "yes",		1 		},
				{ "no",		0 		},
				{ "on",		1 		},
				{ "off",		0 		},
				{ "cr",		'\r'	},
				{ "lf",		'\n'	},
				{ "",			0 		}	/* terminator, must be last keyword */
			};
		

	/*************************************/
	/* Process the file in a single pass */
	/*************************************/
	while (feof(stdin) == 0) /* not end of file */
		{
		if (fgets(line, sizeof(line), stdin) == NULL) /* read failure */
			break;

		lines_read++;

		if (lines_read <= PTSC_MAX_LINES
			&& ftell(stdin) <= PTSC_MAX_FILE_OFFSET) /* first couple pages? */
			{
			/* Make a lowercase copy of the line */
			strncpy(hdr, line, sizeof(hdr)); 
			hdr[sizeof(hdr)-1] = 0;
			for (i=0; hdr[i]; i++)
				hdr[i] = tolower(hdr[i]);

			p = hdr; /* point to beginning of line */

			while ((p = strstr(p, ptsc_header_token)) != NULL) /* token exists */
				{
				if (p != hdr && *(p-1) != ' ' && *(p-1) != '\t')
					{
					p += strlen(ptsc_header_token);	/* skip false header token */
					}
				else /* at beginning of line or immediately after white-space */
					{
					p += strlen(ptsc_header_token);	/* skip valid header token */

					for (i=0; i < PTSC_TOTAL_VARS; i++) /* valid variable name? */
						{
						if (var[i].val_specified == 0
							&& strncmp(p, var[i].name, strlen(var[i].name)) == 0)
							{
							p += strlen(var[i].name);		/* skip variable name	*/
							if (*p == ' ' || *p == '\t')	/* white-space?			*/
								{									/* Yes: valid header		*/
								for (j = 0; j < PTSC_MAX_VALUES; j++)
									{
									/* skip white-space */
									while (*p == ' ' || *p == '\t')	
										p++;

									/* search for keywords */
									for (k = 0; keyword[k].token[0]; k++)
										{
										if (!strncmp(p, keyword[k].token, 
											strlen(keyword[k].token)))
											break;
										}

									if (keyword[k].token[0]) /* keyword used */
										{
										var[i].val[j] = keyword[k].value;
										p += strlen(keyword[k].token);
										continue;
										}

									/* Handle numeric (decimal and hex) values here */
									var[i].val[j] = strtol(p, &np, 0);
									if (np == p || *np == 0)
										break;	/* no more values */
									p = np;
									}
								var[i].val_specified = j;
								}
							break; /* no need to continue searching variable names */
							}
						}
					}
				}
			}
		/************************************************/
		/* Process line here (expand tabs, print, etc.) */
		/************************************************/
		}

	/******************************************/
	/* Display all specified format variables */
	/******************************************/
	for (i = 0; i < PTSC_TOTAL_VARS; i++)
		{
		if (var[i].val_specified > 0)
			{
			fprintf(stderr, "%-15s: ", var[i].name);
			for (j=0; j < var[i].val_specified && j < PTSC_MAX_VALUES; j++)
				{
				fprintf(stderr,"%d ", var[i].val[j]);
				}
			fprintf(stderr, "\n");
			}
		}

}
